﻿Rem
	Player & Enemy sollte das selbe Objekt sein
	oder eine Verwandschaft (im OOP Sinn) haben.
	Das würde ein paar Effekte leichter und variable
	implementierbar machen.
Endrem

Type TPlayerClass
	Field Name:String
	Field Symbol:Int
	Field txt:String
	Field HP_dice:TDice
	Field wpns:String[]
	'Field WC:TWeaponClass[2]
	Field Ziel:TDice, Tarn:TDice, eva:TDice
	Field reg:TDice, lern:TDice
	
	Function Create:TPlayerClass(_Path:String)
		Local file:TStream = ReadFile(_path)
		If file Then
			Local P:TPlayerClass = New TPlayerClass
			P.Name = file.ReadLine()
			P.Symbol = Int(file.ReadLine())
			P.txt = LoadText(file.ReadLine())
			P.HP_dice = TDice.Create(file.ReadLine())
			P.wpns = file.ReadLine().Split(";")
			P.Ziel = TDice.Create(file.ReadLine())
			P.Tarn = TDice.Create(file.ReadLine())
			P.eva = TDice.Create(file.ReadLine())
			P.reg = TDice.Create(file.ReadLine())
			P.lern = TDice.Create(file.ReadLine())
			file.Close()
			Return P
		Else
			RuntimeError("~nTPlayerClass~nCreate() no~n" + StripDir(_Path))
		End If
	EndFunction
End Type

Type TPlayer
	Field Name:String
	Field Symbol:Int, Color:Int
	Field Class:TPlayerClass
	Field HP_Max:Int, HP:Int
	Field Weapon:TWeapon[2], wpn:Int
	Field Targets:TList = New TList
	Field MSG:String
	
	Field Ziel:Int, Tarn:Int, eva:Int
	
	Field X:Int, Y:Int
	
	Method RollStats()
		Self.HP_Max = Self.Class.HP_dice.Roll()
		Self.HP = Self.HP_Max
		Self.Ziel:Int = Self.Class.Ziel.Roll()
		Self.Tarn:Int = Self.Class.Tarn.Roll()
		Self.eva:Int = Self.Class.eva.Roll()
	End Method
	
	Method Draw()
		ASC_Color(Self.Color & $F)
		DrawImage(Ascii, Self.X * TileSize, Self.Y * TileSize, $DB)
		ASC_Color((Self.Color & $F0) / $10)
		DrawImage(Ascii, Self.X * TileSize, Self.Y * TileSize, Self.Symbol)
		
		DrawASCText(Self.GetHP(), 0, 22)
		DrawASCText(Self.GetWpn(), 8, 22)

		'DrawASCText("{20}Hilfe:", 26, 22)
		DrawASCText("{02}F1", 30, 23)
		
		DrawASCText(MSG, 0, 23)
	End Method
	
	Method Move:Int()
		Local nx:Int = Self.x, ny:Int = Self.Y '* New Position
		If KeyHit(KEY_UP) Or KeyHit(KEY_W) Then
			ny:-1
		ElseIf KeyHit(KEY_DOWN) Or KeyHit(KEY_S) Then
			ny:+1
		ElseIf KeyHit(KEY_LEFT) Or KeyHit(KEY_A) Then
			nx:-1
		ElseIf KeyHit(KEY_RIGHT) Or KeyHit(KEY_D) Then
			nx:+1
		ElseIf KeyHit(KEY_PERIOD) Then '* Punkt: Warten/Ausruhen
			Return True
		EndIf
		If nx = Self.X And ny = Self.Y Then Return False
		
		If nx < 0 Or nx >= Map.XSize Then Return False
		If ny < 0 Or ny >= Map.YSize Then Return False
		
		Select Map.Map[nx, ny].Special
			Case 2 '* Tür Öffnen, nicht bewegen
				Map.Map[nx, ny].Special = 0
				Map.Map[nx, ny].Symbol = Map.Map[nx, ny].Class.Symbols[1]
				TInfoText.Add("{20}" + Map.Map[nx, ny].Class.Name + " geöffnet")
				'Player.MSG = 
				Return True
			Case 0 '* Boden -> Bewegen
				Self.X = nx
				Self.Y = ny
				Player.MSG = ""
				Return True
			Default
				Return False
		End Select
		
		Return False
	End Method
	
	Method ReLoad:Int()
		If Self.wpn = -1 Then Return False
		If KeyHit(KEY_R) Then
			Return Self.Weapon[Self.wpn].ReLoad(True)
		EndIf
		Return False
	End Method
	
	Method SwitchWPN:Int()
		If Self.wpn = -1 Then Return False
		If KeyHit(KEY_Q) Then
			Self.wpn:~1
			Return True
		EndIf
	EndMethod
	
	Method FindTargets()
		Self.Targets.Clear()
		For Local e:TEnemy = EachIn TEnemy.List
			If Map.LightMap[e.X, e.Y] = 2 Then Self.Targets.AddLast(e)
		Next
	EndMethod
	
	Method Fire:Int()
		Rem
			True=Keep
			False=Abort
			-1=Fire!
		Endrem
		Local Enemys:TEnemy[9]
		Local s:String = "?"
		
		Local ez:Int = 0
		For Local e:TEnemy = EachIn Self.Targets
			If (Self.wpn = -1 Or Self.Weapon[Self.wpn].Class.mgz_n = -1) And DistInt(Self.x, Self.Y, e.x, e.Y) > 1 Then Continue
			Enemys[ez] = e
			s:+MakeColor(e.Class.Color)
			s:+MakeSymbol(e.Class.Symbol)
			ez:+1
			If ez = 8 Then Exit
		Next
		
		If KeyHit(KEY_F) Or KeyHit(KEY_SPACE) Or KeyHit(KEY_ESCAPE) Then
			FlushKeys()
			Return False
		EndIf
		For Local i:Int = 0 Until ez
			If KeyHit(KEY_1 + i) Then
				If Self.wpn = -1 Then
					Self.Weapon[0].FireOn(Enemys[i], True)
				Else
					Self.Weapon[Self.wpn].FireOn(Enemys[i], True)
				EndIf
				Return - 1
			EndIf
		Next
		
		If ez > 0 Then
			DrawASCText(s, 0, 23)
		Else
			DrawASCText("Kein Ziel! {02}Esc", 0, 23)
			Return True
		EndIf
		SetScale(0.5, 0.5)
			For Local i:Int = 0 Until ez
				Local x:Int = i + 1
				DrawASCText("{F0}" + x, x, 23)
				DrawASCText("{F0}" + x, Enemys[i].x, Enemys[i].Y)
			Next
		SetScale(1, 1)
		Return True
	EndMethod
	
	Method Regenerate(_reset:Int = 0)
		Global RegCount:Int = 0
		If _reset = -1 Then
			RegCount = 10
		ElseIf _reset = True Then
			RegCount = 0
			Return
		EndIf
		If Self.HP = Self.HP_Max Then Return
		RegCount:+1
		If RegCount >= 10 Then
			RegCount = 0
			Self.HP:+Self.Class.reg.Roll()
			Self.HP = LimitInt(Self.HP, 0, Self.HP_Max)
			TInfoText.Add("{60}Reg. {06}" + Self.Class.reg.dice + "{60} +" + Self.Class.reg.LastRoll + "[03]")
		EndIf
	End Method
	
	Method GetHP:String()
		Return "{60}[03]{F0}" + Self.HP + "{20}/{F0}" + Self.HP_Max
	End Method
	
	Method GetWpn:String()
		
		Local n:String, d:String, m:String, s:String, a:String, c:String = "{F0}"
		Local w:Int = Self.wpn
		If w = -1 Then w = 0
		
		n = Self.Weapon[w].Class.Name
		d = Self.Weapon[w].dmg.dice
		
		If Self.wpn <> - 1 Then
			If Self.Weapon[w].Class.mgz <> - 1 Then
				m = Self.Weapon[w].Mag
				s = Self.Weapon[w].Class.mgz
				a = Self.Weapon[w].Ammo
				If Self.Weapon[w].Mag = 0 Then c = "{60}"
				Return n + " {0F}" + d + "{F0} " + c + m + "{20}/{F0}" + s + "{20}/{10}" + a
			EndIf
		EndIf
		
		Return n + " {0F}" + d
	End Method
	
	Method Look:Int(_Set:Int = False)
		Global LX:Int, LY:Int, BlinkSwitch:Int, BlinkTime:Int
		If _Set Then
			LX = Self.X
			LY = Self.Y
			Return True
		End If
		
		If KeyHit(KEY_X) Or KeyHit(KEY_ESCAPE) Then
			FlushKeys()
			Return False
		ElseIf KeyHit(KEY_UP) Or KeyHit(KEY_W) Then
			LY:-1
		ElseIf KeyHit(KEY_DOWN) Or KeyHit(KEY_S) Then
			LY:+1
		ElseIf KeyHit(KEY_LEFT) Or KeyHit(KEY_A) Then
			LX:-1
		ElseIf KeyHit(KEY_RIGHT) Or KeyHit(KEY_D) Then
			LX:+1
		EndIf
		LX = LimitInt(LX, 0, Map.XSize)
		LY = LimitInt(LY, 0, Map.YSize)
		
		If BlinkTime < MilliSecs() Then
			BlinkTime = MilliSecs() + 400
			BlinkSwitch:~1
		End If
		
		If BlinkSwitch Then
			ASC_Color($D)
			DrawImage(Ascii, LX * TileSize, LY * TileSize, $5F)
		EndIf
		
		Local e:TEnemy
		Local s:String, n:String, c:String
		
		For Local Enemy:TEnemy = EachIn Self.Targets
			If Enemy.x = LX And Enemy.y = LY Then e = Enemy
		Next
		
		If e Then
			c = "{06}"
			s = MakeSymbol(e.Class.Symbol)
			n = e.Class.name
		Else
			If Map.LightMap[LX, LY] > 0 Then
				If Self.X = LX And Self.Y = LY Then
					c = "{D0}"
					s = MakeSymbol(Self.Symbol)
					n = "[03][03][03] Du [03][03][03]"
				Else
					c = MakeVGColor(Map.Map[LX, LY].Color)
					s = MakeSymbol(Map.Map[LX, LY].Symbol)
					n = Map.Map[LX, LY].Class.Name
				EndIf
			Else
				c = "{20}"
				s = "???"
				n = "unerforscht"
			EndIf
		EndIf
				
		DrawASCText(c + s + " " + n, 0, 23)
		Return True
	End Method
	
	Method SetClass(_Class:TPlayerClass)
		Self.Class = _Class
		Self.Symbol = _Class.Symbol
		Self.X = -1
		Self.Y = -1
		
		Local WC:TWeaponClass[2]
		WC[0] = TWeaponClass.GetClass(_Class.wpns[0])
		WC[1] = TWeaponClass.GetClass(_Class.wpns[Rand(1, _Class.wpns.length - 1)])
		If WC[1] = Null Or WC[1] = WC[0] Then
			Self.Weapon[0] = TWeapon.Create(WC[0], True)
			Self.wpn = -1
		Else
			Self.Weapon[0] = TWeapon.Create(WC[0])
			Self.Weapon[1] = TWeapon.Create(WC[1])
			Self.wpn = 0
		EndIf
	EndMethod
	
	Method UnSetClass()
		Self.Class = Null
		Weapon[0] = Null
		Weapon[1] = Null
		Self.HP_Max = 0
		Self.Ziel:Int = 0
		Self.Tarn:Int = 0
		Self.eva:Int = 0
	End Method
	
	Method CheckLoose:Int()
		If Self.HP <= 0 Then Return True
		Return False
	End Method
	
End Type
